@@ -187,6 +187,7 @@ plot.predictSurrosurv <- function(
   xlim,
   ylim,
   mfrow,
+  main,
   ...) {
   # ************************************************************************** #
   if (missing(xlab)) xlab <- 'Treatment effect (HR) on S'
@@ -194,7 +195,7 @@ plot.predictSurrosurv <- function(
   if (missing(exact.models))
     exact.models <- all(sapply(tolower(noSpP(models)), function(m)
       any(tolower(noSpP(names(x))) %in% m)))
-  
+  if (missing(main)) main <- format.methodNames(x)[i]
   w <- attr(x, 'trialSizes')
   if (var(w)) {
     w <- .8 + 3 * (w - min(w)) / (max(w) - min(w))
@@ -256,7 +257,7 @@ plot.predictSurrosurv <- function(
                    col=rgb(.2, .2, .2, .8), lwd = 2, add=TRUE)
            },
            xaxt = 'n', yaxt = 'n',
-           main =  format.methodNames(x)[i], 
+           main =  main, 
            xlab = xlab, ylab = ylab, ...)
       if (show.ste) {
         points(STE[i], 0, col = 2, pch = '|', font = 2)
